#!/usr/bin/env node

/**
 * ZAI MCP Server - AI-to-AI Loop System with Multi-Provider Support
 * Supports OpenRouter, Anthropic, and DeepSeek APIs with automatic failover
 */

import { Server } from '@modelcontextprotocol/sdk/server/index.js';
import { StdioServerTransport } from '@modelcontextprotocol/sdk/server/stdio.js';
import { CallToolRequestSchema, ListToolsRequestSchema } from '@modelcontextprotocol/sdk/types.js';
import { MultiProviderAI } from './multiProviderAI.js';
import { ZAIDataCollector } from './data-collection-system.js';
import { AIVotingManager } from './aiVotingManager.js';

class ZAIMCPServer {
    constructor() {
        this.server = new Server(
            {
                name: 'zai-mcp-server',
                version: '2.0.0',
            },
            {
                capabilities: {
                    tools: {},
                },
            }
        );

        // Initialize AI providers, voting system, and data collection
        try {
            this.multiAI = new MultiProviderAI();
            this.votingManager = new AIVotingManager(this.multiAI);
            this.dataCollector = new ZAIDataCollector();
        } catch (error) {
            console.error('❌ Failed to initialize components:', error);
            throw error;
        }
        
        // Active loops storage
        this.activeLoops = new Map();
        this.loopCounter = 0;
        
        // Agent state management
        this.agentBusy = false;
        this.pendingResponses = [];

        this.setupToolHandlers();
        this.setupErrorHandling();

        // 🆓 FREE VERSION - No license validation required
        console.error('🆓 ZAI MCP Server - FREE VERSION');
        console.error('📊 Data collection enabled for AI training');
        console.error('💡 Help us improve AI by using this free service!');
    }

    setupToolHandlers() {
        // List available tools
        this.server.setRequestHandler(ListToolsRequestSchema, async () => ({
            tools: [
                {
                    name: 'activate_infinite_loop',
                    description: 'Start AI-to-AI improvement loops for continuous development',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Activation message with topic (format: "actloop [topic]")'
                            },
                            aiToAi: {
                                type: 'boolean',
                                description: 'Enable AI-to-AI communication mode',
                                default: true
                            },
                            interval: {
                                type: 'number',
                                description: 'Loop interval in milliseconds',
                                default: 5000
                            },
                            maxIterations: {
                                type: 'number',
                                description: 'Maximum iterations before auto-stop',
                                default: 999999
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'stop_ai_loops',
                    description: 'Stop all active AI-to-AI loops',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Stop command (e.g., "stploop")'
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'list_active_loops',
                    description: 'View running loops',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_provider_status',
                    description: 'Check provider status',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'reset_ai_providers',
                    description: 'Reset failed providers',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_prompts',
                    description: 'Get AI-generated prompts',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Maximum number of prompts to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'acknowledge_agent_response',
                    description: 'Process AI responses',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            loopId: {
                                type: 'string',
                                description: 'The ID of the loop to acknowledge'
                            },
                            agentResponse: {
                                type: 'string',
                                description: 'The agent response text'
                            }
                        },
                        required: ['loopId']
                    }
                }
            ]
        }));

        // Handle tool calls
        this.server.setRequestHandler(CallToolRequestSchema, async (request) => {
            const { name, arguments: args } = request.params;

            try {
                switch (name) {
                    case 'activate_infinite_loop':
                        return await this.handleActivateLoop(args);

                    case 'stop_ai_loops':
                        return await this.handleStopAILoops(args);

                    case 'list_active_loops':
                        return await this.handleListActiveLoops(args);

                    case 'get_ai_provider_status':
                        return await this.handleGetAIProviderStatus(args);

                    case 'reset_ai_providers':
                        return await this.handleResetAIProviders(args);

                    case 'get_ai_prompts':
                        return await this.handleGetAIPrompts(args);

                    case 'acknowledge_agent_response':
                        return await this.handleAcknowledgeAgentResponse(args);

                    default:
                        throw new Error(`Unknown tool: ${name}`);
                }
            } catch (error) {
                return {
                    content: [
                        {
                            type: 'text',
                            text: `Error executing tool ${name}: ${error.message}`
                        }
                    ],
                    isError: true
                };
            }
        });
    }

    async handleActivateLoop(args) {
        const { message, aiToAi = true, interval = 5000, maxIterations = 999999 } = args;

        if (!message || typeof message !== 'string') {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing or invalid message parameter. Please provide a string message with format: "actloop [topic]"'
                }]
            };
        }

        // Check if message matches activation pattern
        if (!message.toLowerCase().startsWith('actloop ')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid activation message format. Please use format: "actloop [topic]" (e.g., "actloop improve my React component")'
                }]
            };
        }

        const topic = message.substring(8).trim();
        if (!topic) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ No topic specified. Please provide a topic after "actloop " (e.g., "actloop improve my React component")'
                }]
            };
        }

        try {
            const loopId = `loop-${++this.loopCounter}`;
            const loop = {
                id: loopId,
                topic,
                aiToAi,
                interval,
                maxIterations,
                currentIteration: 0,
                status: 'running',
                startTime: new Date().toISOString(),
                lastActivity: new Date().toISOString()
            };

            this.activeLoops.set(loopId, loop);

            // Start the AI-to-AI loop
            this.startAILoop(loop);

            return {
                content: [{
                    type: 'text',
                    text: `✅ AI-to-AI loop started successfully!\n\n🔄 Loop ID: ${loopId}\n📝 Topic: ${topic}\n🤖 AI-to-AI Mode: ${aiToAi ? 'Enabled' : 'Disabled'}\n⏱️ Interval: ${interval}ms\n🔢 Max Iterations: ${maxIterations}\n\n🚀 The loop is now running and will continuously improve the topic using AI-to-AI collaboration.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to start loop: ${error.message}`
                }]
            };
        }
    }

    async startAILoop(loop) {
        const runIteration = async () => {
            if (loop.status !== 'running' || loop.currentIteration >= loop.maxIterations) {
                this.activeLoops.delete(loop.id);
                return;
            }

            try {
                loop.currentIteration++;
                loop.lastActivity = new Date().toISOString();

                // Generate AI improvement using multi-provider system
                const prompt = `Improve the following topic: "${loop.topic}".

Current iteration: ${loop.currentIteration}
Provide a specific, actionable improvement suggestion.`;

                const response = await this.multiAI.makeRequest(prompt, {
                    maxTokens: 500,
                    temperature: 0.7
                });

                // Collect data for training
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt,
                    response: response.content,
                    context: {
                        loopId: loop.id,
                        topic: loop.topic,
                        iteration: loop.currentIteration
                    },
                    success: true,
                    responseTime: response.responseTime,
                    loopIteration: loop.currentIteration,
                    model: response.model,
                    provider: response.provider,
                    sessionId: loop.id
                });

                console.error(`[AI-TO-AI] Loop ${loop.id} - Iteration ${loop.currentIteration}: ${response.content.substring(0, 100)}...`);

                // Schedule next iteration
                setTimeout(runIteration, loop.interval);

            } catch (error) {
                console.error(`[AI-TO-AI] Error in loop ${loop.id}:`, error);

                // Collect error data
                await this.dataCollector.collectInteraction({
                    type: 'ai-to-ai',
                    prompt: `Improve: ${loop.topic}`,
                    response: null,
                    context: { loopId: loop.id, topic: loop.topic, iteration: loop.currentIteration },
                    success: false,
                    errors: [error.message],
                    sessionId: loop.id
                });

                // Retry after longer interval
                setTimeout(runIteration, loop.interval * 2);
            }
        };

        // Start the first iteration
        setTimeout(runIteration, 1000);
    }

    async handleStopAILoops(args) {
        const { message } = args;

        if (!message || !message.toLowerCase().includes('stploop')) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Invalid stop command. Please use "stploop" to stop AI-to-AI loops.'
                }]
            };
        }

        const stoppedCount = this.activeLoops.size;

        // Stop all active loops
        for (const [, loop] of this.activeLoops) {
            loop.status = 'stopped';
        }
        this.activeLoops.clear();

        // Flush any pending data
        await this.dataCollector.flushData();

        return {
            content: [{
                type: 'text',
                text: `✅ All AI-to-AI loops stopped successfully!\n\n🛑 Stopped ${stoppedCount} active loop(s)\n📊 Data collection completed for all loops`
            }]
        };
    }

    async handleListActiveLoops() {
        if (this.activeLoops.size === 0) {
            return {
                content: [{
                    type: 'text',
                    text: '📋 No active loops currently running.'
                }]
            };
        }

        const loopsList = Array.from(this.activeLoops.values()).map(loop =>
            `🔄 **${loop.id}**\n` +
            `   📝 Topic: ${loop.topic}\n` +
            `   🔢 Iteration: ${loop.currentIteration}\n` +
            `   ⏱️ Started: ${new Date(loop.startTime).toLocaleString()}\n` +
            `   📊 Status: ${loop.status === 'running' ? '🟢 Active' : '🔴 Stopped'}`
        ).join('\n\n');

        return {
            content: [{
                type: 'text',
                text: `📋 **Active AI-to-AI Loops (${this.activeLoops.size})**\n\n${loopsList}`
            }]
        };
    }

    async handleGetAIProviderStatus() {
        try {
            const status = this.multiAI.getStatus();

            return {
                content: [{
                    type: 'text',
                    text: `🤖 **AI Provider Status**\n\n` +
                          `🎯 **Current Provider:** ${status.currentProvider || 'None'}\n` +
                          `📊 **Success Rate:** ${status.successCount}/${status.totalRequests} (${status.successRate}%)\n` +
                          `⚡ **Available Providers:**\n` +
                          Object.entries(status.providers).map(([, provider]) =>
                              `   ${provider.enabled ? '✅' : '❌'} ${provider.name} (${provider.apiKeys} key(s))`
                          ).join('\n') + '\n\n' +
                          `🔄 **Request Statistics:**\n` +
                          `   Total Requests: ${status.totalRequests}\n` +
                          `   Successful: ${status.successCount}\n` +
                          `   Failed: ${status.errorCount}`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to get provider status: ${error.message}`
                }]
            };
        }
    }

    async handleResetAIProviders() {
        try {
            this.multiAI.resetProviders();

            return {
                content: [{
                    type: 'text',
                    text: `✅ AI providers reset successfully!\n\n🔄 All failed providers have been reset and are available for retry.`
                }]
            };
        } catch (error) {
            return {
                content: [{
                    type: 'text',
                    text: `❌ Failed to reset providers: ${error.message}`
                }]
            };
        }
    }

    async handleGetAIPrompts(args) {
        const { limit = 5 } = args;

        // Generate sample AI prompts for demonstration
        const prompts = [];
        for (let i = 0; i < Math.min(limit, 5); i++) {
            prompts.push({
                id: `prompt-${Date.now()}-${i}`,
                content: `AI-generated improvement suggestion ${i + 1}`,
                timestamp: new Date().toISOString(),
                source: 'ZAI-MCP-Server'
            });
        }

        return {
            content: [{
                type: 'text',
                text: `🤖 **AI-Generated Prompts (${prompts.length})**\n\n` +
                      prompts.map((prompt, index) =>
                          `${index + 1}. **${prompt.id}**\n   ${prompt.content}\n   📅 ${new Date(prompt.timestamp).toLocaleString()}`
                      ).join('\n\n')
            }]
        };
    }

    async handleAcknowledgeAgentResponse(args) {
        const { loopId, agentResponse } = args;

        if (!loopId) {
            return {
                content: [{
                    type: 'text',
                    text: '❌ Missing loopId parameter.'
                }]
            };
        }

        const loop = this.activeLoops.get(loopId);
        if (!loop) {
            // For testing purposes, allow acknowledgment even if loop doesn't exist
            console.log(`⚠️ Loop ${loopId} not found, but acknowledging anyway for testing`);
        }

        // Record the agent response for data collection
        if (agentResponse) {
            await this.dataCollector.collectInteraction({
                type: 'agent-response',
                prompt: `Loop ${loopId} acknowledgment`,
                response: agentResponse,
                context: { loopId, topic: loop?.topic || 'unknown' },
                success: true,
                sessionId: loopId
            });
        }

        return {
            content: [{
                type: 'text',
                text: `✅ Agent response acknowledged for loop ${loopId}.\n\n🔄 Loop will continue with next iteration.`
            }]
        };
    }

    setupErrorHandling() {
        this.server.onerror = (error) => {
            console.error('❌ MCP Server error:', error);
            // Don't exit on MCP errors, just log them
        };

        process.on('SIGINT', async () => {
            console.error('🛑 Shutting down ZAI MCP Server...');

            // Stop all active loops
            for (const loop of this.activeLoops.values()) {
                loop.status = 'stopped';
            }
            this.activeLoops.clear();

            try {
                await this.dataCollector.flushData();
                await this.server.close();
            } catch (error) {
                console.error('❌ Error during shutdown:', error);
            }
            process.exit(0);
        });

        process.on('uncaughtException', (error) => {
            console.error('❌ Uncaught exception:', error);
            // Only exit if it's not a buffer-related error
            if (!error.message.includes('subarray') && !error.message.includes('buffer')) {
                process.exit(1);
            }
        });

        process.on('unhandledRejection', (reason, promise) => {
            console.error('❌ Unhandled rejection at:', promise, 'reason:', reason);
            // Only exit if it's not a buffer-related error
            if (reason && !reason.toString().includes('subarray') && !reason.toString().includes('buffer')) {
                process.exit(1);
            }
        });
    }

    async start() {
        try {
            console.error('🚀 Starting ZAI MCP Server...');
            console.error('🔧 Initializing transport layer...');

            // Don't modify stdin before creating transport
            const transport = new StdioServerTransport();
            console.error('📡 Connecting to MCP transport...');

            await this.server.connect(transport);

            console.error('✅ ZAI MCP Server started successfully');
            console.error('📡 Listening for MCP messages...');
            console.error('🎯 Server ready for tool calls');

        } catch (error) {
            console.error('❌ Failed to start MCP server:', error);
            console.error('Error details:', error.stack);
            process.exit(1);
        }
    }
}

// Start the server
async function main() {
    try {
        console.error('🔧 Initializing ZAI MCP Server...');
        const server = new ZAIMCPServer();
        await server.start();
    } catch (error) {
        console.error('❌ Failed to start ZAI MCP Server:', error);
        console.error('Stack trace:', error.stack);
        process.exit(1);
    }
}

// Run if this is the main module
const isMainModule = import.meta.url === `file://${process.argv[1]}` ||
    process.argv[1]?.endsWith('src/index.js') ||
    process.argv[1]?.endsWith('zai-mcp-server') ||
    process.argv[1]?.includes('zai-mcp-server') ||
    process.argv.some(arg => arg.includes('zai-mcp-server'));

if (isMainModule) {
    main().catch((error) => {
        console.error('❌ Fatal error:', error);
        process.exit(1);
    });
}

export { ZAIMCPServer };
